@@ -4,6 +4,7 @@ This module besapi provides a simple interface to the BigFix REST API.
 
 MIT License
 Copyright (c) 2014 Matt Hansen
+Copyright (c) 2021 James Stewart
 Maintained by James Stewart since 2021
 
 Library for communicating with the BES (BigFix) REST API.
@@ -24,8 +25,9 @@ import urllib.parse
 import lxml.etree
 import lxml.objectify
 import requests
+import urllib3.poolmanager
 
-__version__ = "3.9.5"
+__version__ = "4.0.1"
 
 besapi_logger = logging.getLogger("besapi")
 
@@ -116,44 +118,6 @@ def parse_bes_modtime(string_datetime):
     return datetime.datetime.strptime(string_datetime, "%a, %d %b %Y %H:%M:%S %z")
 
 
-# import urllib3.poolmanager
-# # https://docs.python-requests.org/en/latest/user/advanced/#transport-adapters
-# class HTTPAdapterBiggerBlocksize(requests.adapters.HTTPAdapter):
-#     """custom HTTPAdapter for requests to override blocksize
-#     for Uploading or Downloading large files"""
-
-#     # override init_poolmanager from regular HTTPAdapter
-#     # https://stackoverflow.com/questions/22915295/python-requests-post-and-big-content/22915488#comment125583017_22915488
-#     def init_poolmanager(self, connections, maxsize, block=False, **pool_kwargs):
-#         """Initializes a urllib3 PoolManager.
-
-#         This method should not be called from user code, and is only
-#         exposed for use when subclassing the
-#         :class:`HTTPAdapter <requests.adapters.HTTPAdapter>`.
-
-#         :param connections: The number of urllib3 connection pools to cache.
-#         :param maxsize: The maximum number of connections to save in the pool.
-#         :param block: Block when no free connections are available.
-#         :param pool_kwargs: Extra keyword arguments used to initialize the Pool Manager.
-#         """
-#         # save these values for pickling
-#         self._pool_connections = connections
-#         self._pool_maxsize = maxsize
-#         self._pool_block = block
-
-#         # This doesn't work until urllib3 is updated to a future version:
-#         # updating blocksize to be larger:
-#         # pool_kwargs["blocksize"] = 8 * 1024 * 1024
-
-#         self.poolmanager = urllib3.poolmanager.PoolManager(
-#             num_pools=connections,
-#             maxsize=maxsize,
-#             block=block,
-#             strict=True,
-#             **pool_kwargs,
-#         )
-
-
 def get_action_combined_relevance(relevances: list[str]):
     """Take array of ordered relevance clauses and return relevance string for
     action.
@@ -194,7 +158,7 @@ def get_target_xml(targets=None):
         - Array of Integers: ComputerID
     """
     if targets is None or not targets:
-        logging.warning("No valid targeting found, will target no computers.")
+        besapi_logger.warning("No valid targeting found, will target no computers.")
         # default if invalid:
         return "<CustomRelevance>False</CustomRelevance>"
 
@@ -236,7 +200,7 @@ def get_target_xml(targets=None):
                 + "</ComputerName>"
             )
 
-    logging.warning("No valid targeting found, will target no computers.")
+    besapi_logger.warning("No valid targeting found, will target no computers.")
 
     # default if invalid:
     return "<CustomRelevance>False</CustomRelevance>"
@@ -340,6 +304,52 @@ def get_bes_conn_using_config_file(conf_file=None):
     return None
 
 
+# https://docs.python-requests.org/en/latest/user/advanced/#transport-adapters
+class HTTPAdapterBlocksize(requests.adapters.HTTPAdapter):
+    """Custom HTTPAdapter for requests to override blocksize
+    for Uploading or Downloading large files.
+    """
+
+    def __init__(self, blocksize=1000000, **kwargs):
+        self.blocksize = blocksize
+        super().__init__(**kwargs)
+
+    # override init_poolmanager from regular HTTPAdapter
+    # https://stackoverflow.com/questions/22915295/python-requests-post-and-big-content/22915488#comment125583017_22915488
+    def init_poolmanager(self, connections, maxsize, block=False, **pool_kwargs):
+        """Initializes a urllib3 PoolManager.
+
+        This method should not be called from user code, and is only
+        exposed for use when subclassing the
+        :class:`HTTPAdapter <requests.adapters.HTTPAdapter>`.
+
+        :param connections: The number of urllib3 connection pools to cache.
+        :param maxsize: The maximum number of connections to save in the pool.
+        :param block: Block when no free connections are available.
+        :param pool_kwargs: Extra keyword arguments used to initialize the Pool Manager.
+        """
+        # save these values for pickling
+        self._pool_connections = connections
+        self._pool_maxsize = maxsize
+        self._pool_block = block
+
+        try:
+            self.poolmanager = urllib3.poolmanager.PoolManager(
+                num_pools=connections,
+                maxsize=maxsize,
+                block=block,
+                blocksize=self.blocksize,
+                **pool_kwargs,
+            )
+        except Exception:  # pylint: disable=broad-except
+            self.poolmanager = urllib3.poolmanager.PoolManager(
+                num_pools=connections,
+                maxsize=maxsize,
+                block=block,
+                **pool_kwargs,
+            )
+
+
 class BESConnection:
     """BigFix RESTAPI connection abstraction class."""
 
@@ -353,8 +363,21 @@ class BESConnection:
         self.username = username
         self.session = requests.Session()
         self.session.auth = (username, password)
-        # store info on operator used to login
-        # self.operator_info = {}
+
+        # # configure retries for requests
+        # retry = requests.adapters.Retry(
+        #     total=2,
+        #     backoff_factor=0.1,
+        #     # status_forcelist=[500, 502, 503, 504],
+        # )
+
+        # # mount the HTTPAdapter with the retry configuration
+        # self.session.mount("https://", requests.adapters.HTTPAdapter(max_retries=retry))
+
+        # store if connection user is main operator
+        self.is_main_operator = None
+
+        self.webreports_info_xml = None
 
         # use a sitepath context if none specified when required.
         self.site_path = "master"
@@ -424,11 +447,11 @@ class BESConnection:
             if not validate_xsd(data):
                 err_msg = "data being posted did not validate to XML schema. If expected, consider setting validate_xml to false."
                 if validate_xml:
-                    logging.error(err_msg)
+                    besapi_logger.error(err_msg)
                     raise ValueError(err_msg)
 
                 # this is intended it validate_xml is None, but not used currently
-                logging.warning(err_msg)
+                besapi_logger.warning(err_msg)
 
         self.last_connected = datetime.datetime.now()
         return RESTResult(
@@ -445,11 +468,11 @@ class BESConnection:
             if not validate_xsd(data):
                 err_msg = "data being put did not validate to XML schema. If expected, consider setting validate_xml to false."
                 if validate_xml:
-                    logging.error(err_msg)
+                    besapi_logger.error(err_msg)
                     raise ValueError(err_msg)
 
                 # this is intended it validate_xml is None, but not used currently
-                logging.warning(err_msg)
+                besapi_logger.warning(err_msg)
 
         return RESTResult(
             self.session.put(self.url(path), data=data, verify=self.verify, **kwargs)
@@ -462,6 +485,21 @@ class BESConnection:
             self.session.delete(self.url(path), verify=self.verify, **kwargs)
         )
 
+    def am_i_main_operator(self):
+        """Check if the current user is the main operator user."""
+        if self.is_main_operator is None:
+            try:
+                self.webreports_info_xml = self.get("webreports")
+                self.is_main_operator = True
+            except PermissionError:
+                self.is_main_operator = False
+            except Exception as err:  # pylint: disable=broad-except
+                besapi_logger.error("Error checking if main operator: %s", err)
+                self.is_main_operator = None
+
+        if self.is_main_operator is not None:
+            return self.is_main_operator
+
     def session_relevance_json(self, relevance, **kwargs):
         """Get Session Relevance Results in JSON.
 
@@ -545,7 +583,7 @@ class BESConnection:
         )
         return "\n".join(rel_result_array)
 
-    def login(self):
+    def login(self, timeout=(3, 20)):
         """Do login."""
         if bool(self.last_connected):
             duration_obj = datetime.datetime.now() - self.last_connected
@@ -558,21 +596,24 @@ class BESConnection:
             # default timeout is 5 minutes
             # I'm not sure if this is required
             # or if 'requests' would handle this automatically anyway
-            if int(duration_minutes) > 3:
-                besapi_logger.info("Refreshing Login to prevent timeout.")
-                self.last_connected = None
+            # if int(duration_minutes) > 3:
+            #     besapi_logger.info("Refreshing Login to prevent timeout.")
+            #     self.last_connected = None
 
         if not bool(self.last_connected):
-            result_login = self.get("login")
+            result_login = self.get("login", timeout=timeout)
             if not result_login.request.status_code == 200:
                 result_login.request.raise_for_status()
             if result_login.request.status_code == 200:
                 # set time of connection
                 self.last_connected = datetime.datetime.now()
 
-        # This doesn't work until urllib3 is updated to a future version:
-        # if self.connected():
-        #     self.session.mount(self.url("upload"), HTTPAdapterBiggerBlocksize())
+        # This doesn't work until urllib3 is at least ~v2:
+        if self.last_connected:
+            try:
+                self.session.mount(self.url("upload"), HTTPAdapterBlocksize())
+            except Exception:  # pylint: disable=broad-except
+                pass
 
         return bool(self.last_connected)
 
@@ -737,8 +778,9 @@ class BESConnection:
             besapi_logger.warning("User `%s` Already Exists!", new_user_name)
             return result_user
         besapi_logger.info("Creating User `%s`", new_user_name)
-        _ = self.post("operators", lxml.etree.tostring(xml_parsed))
-        # print(user_result)
+        user_result = self.post("operators", lxml.etree.tostring(xml_parsed))
+        besapi_logger.debug("user creation result:\n%s", user_result)
+
         return self.get_user(new_user_name)
 
     def get_computergroup(self, group_name, site_path=None):
@@ -770,7 +812,11 @@ class BESConnection:
 
         # print(lxml.etree.tostring(xml_parsed))
 
-        _ = self.post(f"computergroups/{site_path}", lxml.etree.tostring(xml_parsed))
+        create_group_result = self.post(
+            f"computergroups/{site_path}", lxml.etree.tostring(xml_parsed)
+        )
+
+        besapi_logger.debug("group creation result:\n%s", create_group_result)
 
         return self.get_computergroup(site_path, new_group_name)
 
@@ -795,7 +841,7 @@ class BESConnection:
             raise ValueError("No file_name specified. Must be at least one character.")
 
         if "Upload not found" in result.text:
-            # print("WARNING: Upload not found!")
+            besapi_logger.debug("WARNING: Upload not found!")
             return None
 
         return result
@@ -848,6 +894,9 @@ class BESConnection:
 
         # Example Header::  Content-Disposition: attachment; filename="file.xml"
         headers = {"Content-Disposition": f'attachment; filename="{file_name}"'}
+        logging.warning(
+            "Uploading `%s` to BigFix Server, this could take a while.", file_name
+        )
         with open(file_path, "rb") as f:
             return self.post(self.url("upload"), data=f, headers=headers)
 
@@ -881,8 +930,8 @@ class BESConnection:
         try:
             content = self.get(resource_url.replace("http://", "https://"))
         except PermissionError as err:
-            logging.error("Could not export item:")
-            logging.error(err)
+            besapi_logger.error("Could not export item:")
+            besapi_logger.error(err)
 
         # item_id = int(resource_url.split("/")[-1])
         # site_name = resource_url.split("/")[-2]
@@ -1142,7 +1191,7 @@ class RESTResult:
             if self.validate_xsd(request.text):
                 self.valid = True
             else:
-                logging.debug(
+                besapi_logger.debug(
                     "INFO: REST API Result does not appear to be XML, this could be expected."
                 )
                 self.valid = False
