@@ -14,6 +14,9 @@ from typing import Union
 
 import besapi
 
+if os.name == "nt":
+    import besapi.plugin_utilities_win
+
 
 # NOTE: This does not work as expected when run from plugin_utilities
 def get_invoke_folder(verbose=0):
@@ -124,6 +127,8 @@ def get_plugin_logging_config(log_file_path="", verbose=0, console=True):
         )
     ]
 
+    logging.addLevelName(99, "SESSION")
+
     # log output to console if arg provided:
     if console:
         handlers.append(logging.StreamHandler())
